#ifndef __3D_H_
#define __3D_H_

#include <sprites.h>

#define DEG2RAD(x) (x*M_PI/180.0f)
#define SCALE_FACTOR 40

#define ZBUFFER_SLOPE 4

#define RENDER_MODE_FLAT 0
#define RENDER_MODE_FLAT_Z 1
#define RENDER_MODE_DOT 2

typedef struct _sColor
{
  unsigned char r,g,b,a;
}sColor;

typedef struct _sVector
{
  float x,y,z,r;
  sColor color;

}sVector;
typedef struct _sFace
{
  int a,b,c;
  sColor color;
  sVector *normal;
  unsigned char visible;
  float l;

}sFace;
typedef struct _sObject
{
  int vertexCount;
  sVector *vertex;
  sVector *tvertex;
  sVector *rvertex;
  sVector *d2vertex;
  int faceCount;
  sFace *face;
  sVector position;
  sVector rotation;
  sVector pivot;

  int visible;
  int renderMode;
  int culling;

}sObject;

typedef struct _sCamera
{
  sVector position;
  sVector rotation;
  sVector pivot;
}sCamera;

typedef struct _sCurve
{
  int count;
  sVector *points;
} sCurve;


typedef struct _sScene
{
  int objectCount;
  sObject *object;
  sCamera camera;
  sColor backgroundColor;
  unsigned int *zbuffer;

}sScene;

sScene *scene;


int  init3dEngine(void);
void close3dEngine(void);

/* Properties */
void moveObject(int id, float x, float y, float z);
void rotateObject(int id, float x, float y, float z);
void pivotObject(int id, float x, float y, float z);
void colorObject(int id, int r, int g, int b);
void moveCamera(float x, float y, float z);
void rotateCamera(float x, float y, float z);
void pivotCamera(float x, float y, float z);

void objectSetInvisible(int id);
void objectSetVisible(int id);

sObject *objectGet(int id);
void objectRenderMode(int id, int mode);
void normalVertex(int objid, int faceid);
void objectNormals(int objid);
void normalizeVector(sVector *vect);
void lightObject(int objid, sVector *light);
void objectEnableCulling(int id);
void objectDisableCulling(int id);


/* Objects stuff */
int  addObject(int vtxCount, int faceCount);
int  addCube(float width, float height, float depth, float x, float y, float z);
int  addGalaxy(int starsCount, float radius);
int addTriObject(unsigned char *name);

void drawObjects(void);
void drawObjectFlat(int id);
void drawObjectFlatZbuffer(int id);
void drawObjectDot(int id);

/* Tranformations */
void transformObjects(void);
void rotateVertex(sVector *tvertex, sVector *vertex, sVector *rotation, sVector *pivot);
void translateVertex(sVector *tvertex, sVector *vertex, sVector *translation);
void projectVertex(sVector *d2vertex, sVector *tvertex);

/* Primitives */
void drawTriangleFlat(int *buffer, sVector *vertex1, sVector *vertex2, sVector *vertex3, sColor *color);
void scanlineFlat(int *buffer, int x1, int x2, int y, int color);
void drawTriangleFlatZBuffer(int *buffer, sVector *vertex1, sVector *vertex2, sVector *vertex3, sColor *color);
inline void scanlineFlatZBuffer(int *buffer, int x1, int x2, int z1, int z2, int y, int color);


/*Postprocessing*/
void Cartoon(unsigned int *buffer);

#endif /*__3D_H_*/
