#include <demo.h>
#include <3d.h>
#include <font.h>

int *font;

int initFont(void)
{
  int error;
  int chars;
  FT_Face face;
  error = FT_Init_FreeType( &library ); 
  if (error) 
    { 
      fprintf(stderr, "Can't initialize font system\n");
      return 0;
    }
  
  error = FT_New_Face( library, "data/erasb.pfb", 0, &face );
  if ( error == FT_Err_Unknown_File_Format ) 
    { 
      printf("Font Error\n");
      return 0;
    } 
  else if ( error ) 
    {  
      printf("B0rken Font Error\n");
      return 0;
    }
  font = malloc(94*sizeof(int));
  error = FT_Set_Char_Size( face,   0,   16*64,  300,  300 ); 
  if ( error ) 
    {
      printf("B0rken Font Error\n");
      return 0;
    }
  error = FT_Set_Pixel_Sizes( face,   0,  65 ); 
  if ( error ) 
    {
      printf("B0rken Font Error\n");
      return 0;
    }

  for(chars=32;chars<126;chars++)
    {
   
      FT_UInt glyph_index; 
      FT_GlyphSlot slot; 
      slot = face->glyph;
      glyph_index = FT_Get_Char_Index( face, chars);
      error = FT_Load_Glyph( face,   glyph_index,  FT_LOAD_DEFAULT );
      error = FT_Render_Glyph( face->glyph,  FT_RENDER_MODE_NORMAL ); 
      
      makeFontTexture(slot, chars-34);
  
    }







  return 1;
}


/* Make a 64x64 texture from a character */
int makeFontTexture(FT_GlyphSlot slot, int id)
{
  unsigned char *tbuf;
  int x,y;
  tbuf = (unsigned char *) malloc(64*64);
  
  memset(tbuf, 0, 64*64);
   for(y=0;y<slot->bitmap.rows;y++)
    {
      for(x=0;x<slot->bitmap.width;x++)
	{
	  tbuf[x+y*64] = slot->bitmap.buffer[x+y*slot->bitmap.width];
	}
    }
  memcpy(fontRender[id].buffer, tbuf, 64*64); 

  fontRender[id].x = slot->bitmap_left;
  fontRender[id].y = slot->bitmap_top;
  fontRender[id].w = slot->bitmap.width;
  fontRender[id].h = slot->bitmap.rows;
  
  free(tbuf);

  return 1;
}




void drawChar(unsigned char tid, int px, int py, float size, float r, float g, float b)
{
  int id = tid - 34;
  int x,y;
  float fx=0, fy=0;
  float inc = 1.0f/size;
  unsigned char c;
  int offset, offset2;
  int offsetx;

  int tsize = 64*(int)size;

  px*=4;
  for(y=0;y<tsize;y++)
    {
      if((y+py < YSIZ) && (y+py > 0))
	{
	  offset = (int)(((int)fy)*(int)64);
	  offset2 = (y+py)*(XSIZ*4);
	  
      
	  for(x=0;x<tsize*4;x+=4)
	    {
	      offsetx = (x+px);

	      if(offsetx > (XSIZ-1)*4)
		break;
	      if(offsetx > 0)
		{
	          c = fontRender[id].buffer[((int)fx)+offset];
		  if(c)
		    {
		      screen[offsetx+offset2+2] = (unsigned char)(c*r*1.0f);
		      screen[offsetx+offset2+1] = (unsigned char)(c*g*1.0f);
		      screen[offsetx+offset2] = (unsigned char)(c*b*1.0f);
		    }
		}
	      fx+=inc;
	    }
	}
     
      fy+=inc;
      fx=0;
    }
  
}

void drawString(unsigned char *string, int px, int py, float size, float r, float g, float b)
{
  int i=0;
  int offset=0;
  int ty=py;
  while(string[i])
    {
      if(string[i]==' ')
	offset+=(32*size);
      else if(string[i] == '\n')
	{
	  offset=0;
	  ty+=(50.0f*size);
	}
      else
	{
	  drawChar(string[i], px+(offset), ty, size,r,g,b);
	  offset+=(fontRender[string[i]-34].w*size*1.0f) + 4;
	}
      i++;
    }
}





sOutline *loadOutline(unsigned char *filename)
{
  sOutline *obj;
  FILE *fp;
  unsigned char line[1024];
  int count=0;
  int i;
  int vcount;
  int curObj=0;
  int curVtx=0;
  float x,y,z;

  fp = fopen(filename, "r+");
  if(fp==NULL)
    {
      printf("Can't load %s\n", filename);
      return NULL;
    }
  obj = malloc(sizeof(sOutline));

  while(!feof(fp))
    {
      fgets(line, 1024, fp);
      if(!strncmp(line, "name", 4))
	{

	  break;
	}
    }
  while(!feof(fp))
    {
      fgets(line, 1024, fp);
      if(!strncmp(line, "kids", 4))
	{
	  sscanf(line, "kids %d\n", &count);
	  obj->objCount = count;
	  obj->obj = malloc(sizeof(sOutlineObj)*count);

	  break;
	}
    }
  
 

  for(i=0;i<count;i++)
    {

      while(!feof(fp))
	{
	  fgets(line, 1024, fp);
	  if(!strncmp(line, "loc", 3))
	    {
	      sscanf(line, "loc %f %f %f\n", &x,&y,&z);
	      break;
	    }
	}

      while(!feof(fp))
	{
	  fgets(line, 1024, fp);
	  if(!strncmp(line, "numvert", 7))
	    {
	      sscanf(line, "numvert %d\n", &vcount);
	      obj->obj[curObj].count = vcount;
	      obj->obj[curObj].points = malloc(sizeof(sVector)*vcount);
	      break;
	    }
	}
      for(curVtx=0;curVtx<obj->obj[curObj].count;curVtx++)
	{
	  fgets(line, 1024, fp);
	  sscanf(line, "%f %f %f\n", 
		 &obj->obj[curObj].points[curVtx].x, 
		 &obj->obj[curObj].points[curVtx].y, 
		 &obj->obj[curObj].points[curVtx].z);

	  obj->obj[curObj].points[curVtx].x +=x;
	  obj->obj[curObj].points[curVtx].y +=y;
	  obj->obj[curObj].points[curVtx].z +=z;

	  obj->obj[curObj].points[curVtx].x++;
	  obj->obj[curObj].points[curVtx].y++;
	  obj->obj[curObj].points[curVtx].z++;
	  obj->obj[curObj].points[curVtx].x/=2.0f;
	  obj->obj[curObj].points[curVtx].y/=2.0f;
	  obj->obj[curObj].points[curVtx].z/=2.0f;


	}
      

     
      curObj++;
      
    }
  
  printf("%d\n", obj->objCount );

  fclose(fp);

  return obj;
}




void printOutline(sOutline *line)
{
  int obj, vtx;
  caca_set_color(CACA_COLOR_WHITE, CACA_COLOR_BLACK);

  for(obj=0;obj<line->objCount;obj++)
    {
      for(vtx=0;vtx<line->obj[obj].count-1;vtx++)
	{
	  
	  caca_draw_thin_line(
			      line->obj[obj].points[vtx].x * caca_get_width(),
			      line->obj[obj].points[vtx].y * caca_get_height(),
			      line->obj[obj].points[vtx+1].x * caca_get_width(),
			      line->obj[obj].points[vtx+1].y * caca_get_height());	
	}

	
	 
    }



}
