#include <demo.h>
#include <interpolation.h>
#include <3d.h>
#include <math.h>

float interpLinear(float start, float end, float timeseek)
{
  if(timeseek<0)
    return start;
  if(timeseek>1)
    return end;
  return start*(1-timeseek)+end*timeseek;
}
float interpCosine(float start, float end, float timeseek)
{
   float time2;

  if(timeseek<0)
    return start;
  if(timeseek>1)
    return end;

   time2 = (1-cos(timeseek*M_PI))/2;
   return(start*(1-time2)+end*time2);
}

float interpCubic(float y0, float y1,float y2, float y3, float timeseek )
{
   float a0,a1,a2,a3,mu2;

   mu2 = timeseek*timeseek;
   a0 = y3 - y2 - y0 + y1;
   a1 = y0 - y1 - a0;
   a2 = y2 - y0;
   a3 = y1;

   return(a0*timeseek*mu2+a1*mu2+a2*timeseek+a3);
}

/* Loads an ac3d file ASSUMING there is one line in it */
/* Returns a sCurve pointer with all points */

sCurve *loadCurve(unsigned char *filename)
{
  FILE *fp;
  sVector *line;
  sCurve *curve;
  int found=0;
  int number=0;
  int i;
  unsigned char data[1024];

  fp = fopen(filename, "r+");
  if(fp==NULL)
    {
      fprintf(stderr, "Can't open %s for reading\n", filename);
      return NULL;
    }

  while(!feof(fp))
    {
      fgets(data, 1024, fp);
      if(!strncmp(data, "numvert",7))
	{
	  found = 1;
	  break;
	}
    }
  if(!found)
    {
      fprintf(stderr, "Can't find any line on %s\n", filename);
      return NULL;
    }

  sscanf(data, "numvert %d\n", &number);

  line = malloc(sizeof(sVector)*number);
  curve= malloc(sizeof(sCurve));

 
  for(i=0;i<number;i++)
    {
      fgets(data, 1024, fp);
      sscanf(data, "%f %f %f\n", &line[i].x,&line[i].y,&line[i].z);

      line[i].x*=SCALE_FACTOR;
      line[i].y*=SCALE_FACTOR;
      line[i].z*=SCALE_FACTOR;


     
    }

  fclose(fp);



  curve->count = number;
  curve->points = line;
  return curve;
}



void displayCurve(sCurve *curve)
{
  int i;
  for(i=0;i<curve->count;i++)
    {
      printf("\t%f %f %f\n",
	     curve->points[i].x,
	     curve->points[i].y,
	     curve->points[i].z);
    }
}



void getCurvePosition(sCurve *curve, sVector *position, float timeseek, int type)
{
  float t = timeseek*curve->count*1.0f;
  float t2 = t - ((int)t);
  
  
  if(t>(curve->count-1))
    {
      position->x = curve->points[curve->count-1].x;
      position->y = curve->points[curve->count-1].y;
      position->z = curve->points[curve->count-1].z;
      return;
    }
  if(t<0)
    {
      position->x = curve->points[0].x;
      position->y = curve->points[0].y;
      position->z = curve->points[0].z;
      return;
    }
  
  if(type==0)
    {
      position->x = interpCosine(curve->points[(int)t].x, curve->points[(int)t+1].x, t2);
      position->y = interpCosine(curve->points[(int)t].y, curve->points[(int)t+1].y, t2);
      position->z = interpCosine(curve->points[(int)t].z, curve->points[(int)t+1].z, t2);
    }
  else if(type==1)
    {
      position->x = interpLinear(curve->points[(int)t].x, curve->points[(int)t+1].x, t2);
      position->y = interpLinear(curve->points[(int)t].y, curve->points[(int)t+1].y, t2);
      position->z = interpLinear(curve->points[(int)t].z, curve->points[(int)t+1].z, t2);
    }
  else
    {
      if(t>(curve->count-3))
	{
	  position->x = curve->points[curve->count-1].x;
	  position->y = curve->points[curve->count-1].y;
	  position->z = curve->points[curve->count-1].z;
	  return; 
	}


      position->x = interpCubic(curve->points[(int)t].x, curve->points[(int)t+1].x, 
				curve->points[(int)t+2].x, curve->points[(int)t+3].x, 
				t2);
      position->y = interpCubic(curve->points[(int)t].y, curve->points[(int)t+1].y, 
				curve->points[(int)t+2].y, curve->points[(int)t+3].y, 
				t2);
      position->z = interpCubic(curve->points[(int)t].z, curve->points[(int)t+1].z, 
				curve->points[(int)t+2].z, curve->points[(int)t+3].z, 
				t2);

    }
  
}



