#include <demo.h>
#include <3d.h>
#include <particles.h>
#include <font.h>
#include <sprites.h>

sParticleEmitter *emiter=NULL;

int emiterCount=0;

int initParticles(int count, float lifetime, float max_speed, float gravity, float up_speed, float far, unsigned char *sprite)
{
  int i;
  
  if(emiter==NULL)
    {
      emiter = malloc(sizeof(sParticleEmitter));
      for(i=0;i<RAND_TABLE_MAX;i++)
	rand_table[i] = (((float)rand()/RAND_MAX));
      rand_offset = 0;
    }
  else
    emiter = realloc(emiter, sizeof(sParticleEmitter) * (emiterCount+1));
  
  emiter[emiterCount].sprite = loadPng(sprite);
  if(emiter[emiterCount].sprite == NULL)
    {
      fprintf(stderr, "Error while loading '%s'\n", sprite);
      return 0;
    }

  emiter[emiterCount].nbParticle = count;
  emiter[emiterCount].lifetime = lifetime;
  emiter[emiterCount].max_speed = max_speed;
  emiter[emiterCount].gravity = gravity;
  emiter[emiterCount].up_speed = up_speed;
  emiter[emiterCount].far = far;

  emiter[emiterCount].particles = malloc(sizeof(sParticle) * count);
  emiter[emiterCount].init = 0;

  for(i=0;i<emiter[emiterCount].nbParticle;i++)
    {
      emiter[emiterCount].particles[i].visible=0;
    }

  emiter[emiterCount].objectId = addObject(count, 0);
  emiter[emiterCount].object = objectGet(emiter[emiterCount].objectId);

  objectSetInvisible(emiter[emiterCount].objectId);

  emiterCount++;
  return emiterCount-1;  
}


void updateParticles(int id, float x, float y, float z,int ticks)
{
  int i;
  if(!emiter[id].init)
    {
      for(i=0;i<emiter[id].nbParticle;i++)
	{

	  if(emiter[id].particles[i].visible)
	    {
	      regenerateParticle(id, i,x,y,z);
	    }
	}
      emiter[id].init = 1;
    }
  

  for(i=0;i<emiter[id].nbParticle;i++)
    {
      emiter[id].particles[i].x+=(emiter[id].particles[i].vx*emiter[id].particles[i].speed);
      emiter[id].particles[i].y+=(emiter[id].particles[i].vy*emiter[id].particles[i].speed);
      emiter[id].particles[i].z+=(emiter[id].particles[i].vz*emiter[id].particles[i].speed);
	  
      emiter[id].particles[i].life-=1;
  
 
      if((emiter[id].particles[i].life<=0) || (!emiter[id].particles[i].visible))
	{
	  regenerateParticle(id, i,x,y,z);	
	}
      emiter[id].particles[i].vy-=emiter[id].gravity;
    }

  convertParticleToObject(id);
}


void regenerateParticle(int id, int part, float x, float y, float z)
{
  emiter[id].particles[part].ex = x;
  emiter[id].particles[part].ey = y;
  emiter[id].particles[part].ez = z;
  
  emiter[id].particles[part].x = 0;
  emiter[id].particles[part].y = 0;
  emiter[id].particles[part].z = 0;
  
  emiter[id].particles[part].vx = (GET_RAND()-0.5)*emiter[id].far;
  emiter[id].particles[part].vy = (GET_RAND()-0.5)*emiter[id].up_speed;
  emiter[id].particles[part].vz = (GET_RAND()-0.5)*emiter[id].far;
  emiter[id].particles[part].speed =(2.0f+(GET_RAND() )*emiter[id].max_speed);
  emiter[id].particles[part].life = (GET_RAND())*emiter[id].lifetime; 

  emiter[id].particles[part].visible = 1;
}


void convertParticleToObject(int id)
{
  int i;

  emiter[id].object = objectGet(emiter[id].objectId);

 

  for(i=0;i<emiter[id].nbParticle;i++)
    {
      emiter[id].object->vertex[i].x = emiter[id].particles[i].x + emiter[id].particles[i].ex;
      emiter[id].object->vertex[i].y = emiter[id].particles[i].y + emiter[id].particles[i].ey;
      emiter[id].object->vertex[i].z = emiter[id].particles[i].z + emiter[id].particles[i].ez;
    }
}
sObject *particleGetObject(int id)
{
  return emiter[id].object;
}
int particleGetObjectId(int id)
{
  return emiter[id].objectId;
}


void drawParticles(int id)
{
  int i;

  for(i=0;i<emiter[id].nbParticle;i++)
    {
      putSpriteAdditiveZBuffer((int *)screen,emiter[id].sprite , 
			       emiter[id].object->d2vertex[i].x,
			       emiter[id].object->d2vertex[i].y
			       ,emiter[id].object->d2vertex[i].z , 
			       (emiter[id].particles[i].life/emiter[id].lifetime));
    }
  
}


/* Create 3d particle object from ac3d file */
sSystem *createParticlesFromObject(unsigned char *filename)
{
  sSystem *sys;
  sCurve *curve;
  int i;

  curve = loadCurve(filename);
  sys = malloc(sizeof(sSystem));
  sys->particles = malloc(sizeof(int)*curve->count);

  for(i=0;i<curve->count;i++)
    {
      sys->particles[i] = initParticles(5, 10, 0.1,0.001,-1.0, 0.5, "data/particle.png");
 /*int count, float lifetime, float max_speed, float gravity, float up_speed, float far*/


    }
  printf("%d particles system created\n", curve->count);
  sys->count = curve->count;
  sys->curve=curve;

  return sys;
}





/* No need for a real random number at all */
inline float GET_RAND(void)
{
  rand_offset++;
  if(rand_offset>RAND_TABLE_MAX)
    rand_offset=0;
  return rand_table[rand_offset];
}
